﻿Imports System.ComponentModel

''' <summary>
''' Zeichnet ein Rubberband auf einem Control und liefert den ausgewählten 
'''  Bereich zurück
''' </summary>
Public NotInheritable Class Rubberband

    ''' <summary>
    ''' Enumeration welche die möglichen Stile auflistet
    ''' </summary>
    Public Enum Styles
        ''' <summary>
        ''' Zeichnet einen gestrichelten Rahmen
        ''' </summary>
        Frame = 0
        ''' <summary>
        ''' Zeichnet ein gefülltes Rechteck
        ''' </summary>
        Rectangle = 1
        ''' <summary>
        ''' Zeichnet eine gestrichelte Linie
        ''' </summary>
        Line = 2
    End Enum

    ''' <summary>
    ''' Enumeration welche die möglichen Modi auflistet
    ''' </summary>
    ''' <remarks></remarks>
    Public Enum Modes
        ''' <summary>
        ''' Rubberband wird nach drücken der Maustaste/n gestartet und durch 
        '''  loslassen wieder beendet
        ''' </summary>
        ''' <remarks></remarks>
        MouseDown = 0
        ''' <summary>
        ''' Rubberband wird durch einen Click gestartet und durch einen 
        '''  weiteren Click beendet
        ''' </summary>
        ''' <remarks>In diesem Modus kann eine Maustaste definiert werden durch 
        '''  die ein Abbruch statt findet und kein Finished-Ereignis ausgelöst  
        '''  wird
        '''</remarks>
        Click = 1
    End Enum

    ' Lokale Varable die den eingestellten Mode speichert
    Private _Mode As Modes
    ' lokale Variable die den eingestellen Stil speichert
    Private _Style As Styles = Styles.Frame
    ' darf ich aktiv werden?
    Private _IsActive As Boolean = False
    ' darf das Rubberband nur im ClientRectangle des Controls gezeichnet werden?
    Private _ClientRectOnly As Boolean = True
    ' Auf welche Maustasten soll reagiert werden?
    Private _Buttons As MouseButtons
    ' Maustaste/n bei der/denen die Aktion im Modus 'Click' abgebrochen werden 
    '  soll
    Private _CancelButtons As MouseButtons
    ' wurde ein Gummiband gezeichnet?
    Private _visible As Boolean
    ' Ausgangspunkt für das Rubberband
    Private _RubberOrigin As Point
    ' Endpunkt für das Rubberband
    Private _RubberCurrent As Point
    ' Optionales Begrenzungsrechteck für das Rubberband
    Private _Rect As Rectangle
    ' Hintergrundfarbe für das Rubberband im Rechteck Modus
    Private _Backcolor As Color = Color.Yellow
    ' Das Control auf dem das Rubberband gezeichnet werden soll
    Private WithEvents _Control As Control

    ''' <summary>
    ''' Ereignis welches die Beendigung von Rubberband meldet und die 
    '''  Koordinaten liefert
    ''' </summary>
    ''' <param name="sender">
    '''  Das Rubberband-Objekt von dem das Ereignis ausgelöst wurde
    ''' </param>
    ''' <param name="e">
    '''  Ein RubberbandEventArgs-Objekt mit den Koordinaten des ausgewählten 
    '''   Bereichs und er gedrückten Maustasten
    ''' </param>
    Public Event Finished(ByVal sender As Object, _
                          ByVal e As RubberBandEventArgs)

    ''' <summary>
    ''' Ereignis welches den Abbruch von Rubberband meldet
    ''' </summary>
    ''' <param name="sender">
    '''  Das Rubberband-Objekt von dem das Ereignis ausgelöst wurde
    ''' </param>
    ''' <param name="e">Ein leeres EventArgs-Objekt</param>
    ''' <remarks></remarks>
    Public Event Aborted(ByVal sender As Object, ByVal e As EventArgs)

    ''' <summary>
    ''' Initialisiert eine neue Instanz eines Rubberband
    ''' </summary>
    Public Sub New()
    End Sub

    ''' <summary>
    ''' Initialisiert eine neue Instanz eines Rubberband
    ''' </summary>
    ''' <param name="Control">
    '''  Ein von Control abgeleitetes Objekt auf dessen MausDown-Ereignis 
    '''  reagiert werden soll
    '''</param>
    Public Sub New(ByVal Control As Control)
        _Control = Control
    End Sub

    ''' <summary>
    ''' Legt das von Control abgeleitete Objekt fest auf dessen MausDown-
    '''  Ereignis reagiert werden soll oder gibt es zurück
    ''' </summary>
    Public Property Control() As Control
        Get
            Return _Control
        End Get
        Set(ByVal value As Control)
            _Control = value
        End Set
    End Property

    ''' <summary>
    ''' Legt den Stil fest oder gibt ihn zurück
    ''' </summary>
    Public Property Style() As Styles
        Get
            Return _Style
        End Get
        Set(ByVal value As Styles)
            _Style = value
        End Set
    End Property

    ''' <summary>
    ''' Legt die Füllfarbe für den Modus Rechteck fest oder gibt sie zurück
    ''' </summary>
    ''' <remarks>
    '''  Entspricht der angezeigten Farbe wenn das Rechteck über einem schwarzen 
    '''   Hintergrund liegt
    ''' </remarks>
    Public Property BackColor() As Color
        Get
            Return _Backcolor
        End Get
        Set(ByVal value As Color)
            _Backcolor = value
        End Set
    End Property

    ''' <summary>
    ''' Legt einen Wert fest der bestimmt ob das Rubberband auf den 
    '''  Clientbereich des Controls begrenzt sein soll, oder gibt ihn zurück
    ''' </summary>
    ''' <value>True wenn eine Begrenzung erfolgen soll, sonst False</value>
    Public Property ClientRectOnly() As Boolean
        Get
            Return _ClientRectOnly
        End Get
        Set(ByVal value As Boolean)
            _ClientRectOnly = value
        End Set
    End Property

    ''' <summary>
    ''' Legt ein Begrenzungsrechteck fest oder gibt es zurück
    ''' </summary>
    Public Property RestrictionRect() As Rectangle
        Get
            If _Rect.IsEmpty Then
                Return _Control.ClientRectangle
            Else
                Return _Rect
            End If
        End Get
        Set(ByVal value As Rectangle)
            _Rect = value
        End Set
    End Property

    ''' <summary>
    ''' Liefert True wenn Rubberband auf Maustasten reagieren soll.
    ''' </summary>
    Public ReadOnly Property IsActive() As Boolean
        Get
            Return _IsActive
        End Get
    End Property

    ''' <summary>
    ''' Legt einen Wert für den Modus fest oder gibt ihn zurück
    ''' </summary>
    ''' <value>Ein gülter Modes-Wert </value>
    ''' <remarks></remarks>
    Public Property Mode() As Modes
        Get
            Return _Mode
        End Get
        Set(ByVal value As Modes)
            _Mode = value
        End Set
    End Property

    ''' <summary>
    ''' Setzt Rubberband in den Aktiv-Modus und legt die Maustasten fest auf 
    '''  die reagiert werden soll
    ''' </summary>
    ''' <param name="Button">
    '''  Ein Wert aus der MouseButtons-Enumeration der die Maustaste/n bestimmt 
    '''  auf die reagiert werden soll
    ''' </param>
    ''' <param name="CancelButton">
    '''  Ein Wert aus der MouseButtons-Enumeration der die Maustaste/n bestimmt 
    '''  mit der Rubberband abgerochen wird
    ''' </param>
    Public Sub Start(ByVal Button As MouseButtons, _
                     ByVal CancelButton As MouseButtons)
        _Buttons = Button
        _CancelButtons = CancelButton
        _IsActive = True
    End Sub

    ''' <summary>
    ''' Bricht den Aktiv-Modus ab und beendet Rubberband
    ''' </summary>
    ''' <remarks>Es wird kein Abort-Ereignis ausgelöst</remarks>
    Public Sub Abort()
        If _IsActive Then
            If _visible Then
                DrawRubberband()
                _visible = False
            End If
            _IsActive = False
        End If
    End Sub

    Private Sub _Control_MouseClick(ByVal sender As Object, _
            ByVal e As System.Windows.Forms.MouseEventArgs) _
            Handles _Control.MouseClick

        If _Mode = Modes.Click Then
            If _IsActive Then
                If _visible = True Then
                    ' Rubberband löschen
                    DrawRubberband()
                    _visible = False
                    If (e.Button And _Buttons) > MouseButtons.None Then
                        ' Rubberband wurde korrekt beendet
                        RaiseEvent Finished(Me, _
                            New RubberBandEventArgs(_RubberOrigin, _
                                                    _RubberCurrent, e.Button))
                    End If
                    If (e.Button And _CancelButtons) > MouseButtons.None Then
                        ' Rubberband wurde abgebrochen
                        RaiseEvent Aborted(Me, New EventArgs)
                    End If
                    ' nicht mehr auf Mause-Ereignisse reagieren
                    _IsActive = False
                Else
                    If (e.Button And _Buttons) > MouseButtons.None Then
                        ' Startpunkt merken
                        _RubberOrigin = e.Location
                        ' aktuelle Mausposition speichern
                        _RubberCurrent = e.Location
                        ' Rubberband anzeigen
                        DrawRubberband()
                        ' merken das Rubberband gezeichnet wurde
                        _visible = True
                    End If
                End If
            End If
        End If
    End Sub

    Private Sub _Control_MouseDown(ByVal sender As Object, _
            ByVal e As System.Windows.Forms.MouseEventArgs) _
            Handles _Control.MouseDown

        If _Mode = Modes.MouseDown Then
            ' Ist Rubberband aktiv und einer der richtigen MouseButton gedrückt?
            If _IsActive And (_Buttons And e.Button) <> MouseButtons.None Then
                ' Startpunkt merken
                _RubberOrigin = e.Location
                ' aktuelle Mausposition speichern
                _RubberCurrent = e.Location
                ' Rubberband anzeigen
                DrawRubberband()
                ' merken das Rubberband gezeichnet wurde
                _visible = True
            End If
        End If
    End Sub

    Private Sub _Control_MouseMove(ByVal sender As Object, _
            ByVal e As System.Windows.Forms.MouseEventArgs) _
            Handles _Control.MouseMove

        If _Mode = Modes.MouseDown Then
            ' Ist Rubberband aktiv und einer der richtige Button gedrückt?
            If _IsActive And (e.Button And _Buttons) <> MouseButtons.None And _
                _visible Then

                ' vorher gezeichnetes Rubberband löschen
                DrawRubberband()
                ' aktuelle Mausposition speichern
                _RubberCurrent = CalcPoint(e.Location)
                ' Rubberband wieder anzeigen
                DrawRubberband()
            End If
        ElseIf _Mode = Modes.Click Then
            If _visible = True Then
                ' vorher gezeichnetes Rubberband löschen
                DrawRubberband()
                ' aktuelle Mausposition speichern
                _RubberCurrent = CalcPoint(e.Location)
                ' Rubberband wieder anzeigen
                DrawRubberband()
            End If
        End If
    End Sub

    Private Sub _Control_MouseUp(ByVal sender As Object, _
            ByVal e As System.Windows.Forms.MouseEventArgs) _
            Handles _Control.MouseUp

        If _Mode = Modes.MouseDown Then
            ' Ist Rubberband aktiv und einer der richtige Button gedrückt?
            If _IsActive And (e.Button And _Buttons) <> MouseButtons.None Then
                ' Rubberband löschen falls eins gezeichnet wurde
                If _visible Then
                    DrawRubberband()
                End If
                ' nicht mehr auf Mause-Ereignisse reagieren
                _IsActive = False
                ' Über die korrekte Fertigstellung benachrichtigen und den 
                '  ausgewählten Bereich zurück liefern
                RaiseEvent Finished(Me, _
                                    New RubberBandEventArgs(_RubberOrigin, _
                                                            _RubberCurrent, _
                                                            e.Button))
            End If
        End If
    End Sub

    ''' <summary>
    ''' Liefert abhängig von den Einstellungen für RestrictionRect und
    '''  ClientRectOnly gültiges Point-Objekt 
    ''' </summary>
    ''' <param name="pt">Point mit den Mauskoordinaten</param>
    ''' <remarks>RestrictionRect hat immer Vorrang vor ClientRectOnly</remarks>
    Private Function CalcPoint(ByVal pt As Point) As Point
        ' Auf das ClientRectangle des Controls begrenzen?
        Dim rct As Rectangle
        If ClientRectOnly Then
            If _Rect.IsEmpty Then
                rct = _Control.ClientRectangle
            Else
                rct = _Rect
            End If
        End If
        If Not rct.IsEmpty Then
            If pt.X < _Control.ClientRectangle.Left Then
                pt.X = _Control.ClientRectangle.Left
            End If
            If pt.X > _Control.ClientRectangle.Right Then
                pt.X = _Control.ClientRectangle.Right
            End If
            If pt.Y < _Control.ClientRectangle.Top Then
                pt.Y = _Control.ClientRectangle.Top
            End If
            If pt.Y > _Control.ClientRectangle.Bottom Then
                pt.Y = _Control.ClientRectangle.Bottom
            End If
        End If
        Return pt
    End Function

    ''' <summary>
    ''' Zeichnet auf den Screen oder löscht ein vorher gezeichnetes Objekt.
    ''' </summary>
    ''' <remarks>
    '''  Es wir im XOR-Modus auf den Screen gezeichnet, daher wird eine 
    '''  vorherige Zeichnung an geleicher Stelle wieder umgekehrt.
    ''' </remarks>
    Private Sub DrawRubberband()
        ' Koordinaten der Punkte von Client-Koordinaten in Screen-Koordinaten
        '  umrechnen
        Dim pt1 As Point = _Control.PointToScreen(_RubberOrigin)
        Dim pt2 As Point = _Control.PointToScreen(_RubberCurrent)
        ' Rechteck für die Stile 'Frame' und 'Rectangle' erstellen
        Dim Rect As New Rectangle(pt1, New Size(pt2.X - pt1.X, pt2.Y - pt1.Y))
        Select Case _Style
            Case Styles.Frame
                ControlPaint.DrawReversibleFrame(Rect, Color.Transparent, _
                                                 FrameStyle.Dashed)
            Case Styles.Rectangle
                ControlPaint.FillReversibleRectangle(Rect, _Backcolor)
            Case Styles.Line
                ControlPaint.DrawReversibleLine(pt1, pt2, Color.Transparent)
        End Select
    End Sub
End Class
